@@ -12,6 +12,7 @@
 /// the branch predictor behavior to trade average performance for a more predictable execution time
 pub mod mbpm {
     use bit_field::BitField;
+    use core::arch::asm;
     /// Branch prediction mode register
     #[derive(Clone, Copy, Debug)]
     #[repr(transparent)]
@@ -39,12 +40,12 @@ pub mod mbpm {
     /// Set mode to dynamic direction prediction.
     #[inline]
     pub unsafe fn clear_bdp() {
-        asm!("csrrci 0x7C0, 0")
+        asm!("csrci 0x7C0, 0")
     }
     /// Set mode to static-taken direction prediction.
     #[inline]
     pub unsafe fn set_bdp() {
-        asm!("csrrsi 0x7C0, 0")
+        asm!("csrsi 0x7C0, 0")
     }
 }
 
@@ -68,11 +69,12 @@ pub mod mbpm {
 /// on documentation of each core.
 pub mod mfeature {
     use crate::feature::Mask;
+    use core::arch::asm;
 
     /// Clear corresponding bits in feature register
     #[inline]
     pub unsafe fn clear_features(flags: Mask) {
-        asm!("csrrc 0x7C1, {}", in(reg) flags.bits())
+        asm!("csrc 0x7C1, {}", in(reg) flags.bits())
     }
 }
 
@@ -81,6 +83,7 @@ pub mod mfeature {
 /// The mnscratch CSR holds a 64-bit read-write register, which enables the NMI trap handler
 /// to save and restore the context that was interrupted.
 pub mod mnscratch {
+    use core::arch::asm;
     /// Reads the `mnscratch` register
     #[inline]
     pub fn read() -> usize {
@@ -101,6 +104,7 @@ pub mod mnscratch {
 /// holds the PC of the instruction that took the interrupt. The lowest bit of mnepc is
 /// hardwired to zero.
 pub mod mnepc {
+    use core::arch::asm;
     /// Reads the `mnepc` register
     #[inline]
     pub fn read() -> usize {
@@ -123,6 +127,7 @@ pub mod mnepc {
 /// | 2 | RNMI input pin | External `rnmi_N` input |
 /// | 3 | Bus error | RNMI caused by BEU |
 pub mod mncause {
+    use core::arch::asm;
     /// NMI causes
     #[repr(usize)]
     pub enum Nmi {
@@ -156,4 +161,7 @@ pub mod mncause {
 /// The mnstatus CSR holds a two-bit field, which, on entry to the trap handler,
 /// holds the privilege mode of the interrupted context encoded in the same manner
 /// as mstatus.mpp.
-pub mod mnstatus {}
+pub mod mnstatus {
+    // FIXME: this register is located at CSR 0x353, but the specifications did not give
+    // how the bits in this register is defined.
+}
